//
//  App.swift
//  MinGraviditet
//
//  Created by Thomas Kalhøj Clemensen on 08/10/2020.
//

import Foundation
import TriforkSwiftDependencyInjection
import TriforkSwiftLogger

enum BuildConfiguration: String {
    case debug = "Debug"
    case test = "Test"
    case testMock = "TestMock"
    case release = "Release"
    case unknown = "Unknown"
}

struct Configuration {
    let backendUrl: String
    let useMock: Bool
    let loggerConfig: TriforkLoggerConfig
}

final class App {
    static let buildConfiguration = BuildConfiguration(rawValue: Bundle.main.buildConfiguration) ?? .unknown
    static let versionString: String = Bundle.main.versionString
    static let configuration = Configuration.build(for: buildConfiguration)

    private static let multiLogger = MultiLogger(loggers: [])

    static func buildDependencies() {
        setupLoggers()

        Resolver.shared.register(LoggerProtocol.self, { multiLogger })

        if configuration.useMock {
            addMockServices()
        } else {
            addServices()
        }
    }

    private static func setupLoggers() {
        var loggers: [LoggerProtocol] = [TriforkLogger(config: configuration.loggerConfig)]
        if buildConfiguration == .debug {
            loggers.append(ErrorFlashLogger())
        }
        multiLogger.loggers = loggers
    }

    private static func addServices() {
        // Use TriforkSwiftDependencyInjection
    }

    private static func addMockServices() {
        // Use TriforkSwiftDependencyInjection
    }
}

private extension Configuration {
    static func build(for buildConfiguration: BuildConfiguration) -> Configuration {
        switch buildConfiguration {
        case .debug:
            return Configuration(
                backendUrl: "",
                useMock: true,
                loggerConfig: TriforkLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .default,
                    isDevelopmentInfoEnabled: false,
                    isEmojisEnabled: true
                )
            )
        case .test:
            return Configuration(
                backendUrl: "",
                useMock: false,
                loggerConfig: TriforkLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .info,
                    isDevelopmentInfoEnabled: true,
                    isEmojisEnabled: true
                )
            )
        case .testMock:
            return Configuration(
                backendUrl: "",
                useMock: true,
                loggerConfig: TriforkLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .info,
                    isDevelopmentInfoEnabled: true,
                    isEmojisEnabled: true
                )
            )
        case .release:
            return Configuration(
                backendUrl: "",
                useMock: false,
                loggerConfig: TriforkLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .error,
                    isDevelopmentInfoEnabled: true,
                    isEmojisEnabled: true
                )
            )
        case .unknown:
            fatalError("App was started with unknown build configuration 🤯")
        }
    }
}
